@@ -3,12 +3,14 @@ This module provides a class `Visualizer` to visualize a `ConceptLattice`
 """
 from fcapy.poset import POSet
 from fcapy.lattice import ConceptLattice
-from fcapy.visualizer.layouts import LAYOUTS
+from fcapy.visualizer.line_layouts import LAYOUTS
 from fcapy.utils.utils import get_kwargs_used
 
 import networkx as nx
 from collections.abc import Iterable
 
+import warnings
+
 
 class POSetVisualizer:
     """
@@ -48,6 +50,12 @@ class POSetVisualizer:
         label_font_size: `int`
             The size of a font size when labeling the nodes
         """
+        warnings.warn(
+            "The use of class POSetVisualizer and its successors is deprecated and will be removed in future versions."
+            "Please, move to use LineVizNx class for visualization",
+            FutureWarning
+        )
+
         assert poset is not None, "Cannot visualize an empty poset"
 
         self._poset = poset
@@ -82,6 +90,7 @@ class POSetVisualizer:
         nodelist:list = None
     ):
         """Draw line diagram of the `POSet` with `networkx` package
+
         Parameters
         ----------
         draw_node_indices: `bool`
@@ -106,7 +115,7 @@ class POSetVisualizer:
             if e[0] not in missing_nodeset and e[1] not in missing_nodeset
         ]
 
-        cs = f'arc3,rad={edge_radius}' if edge_radius is not None else None
+        cs = f'arc3,rad={edge_radius}' if edge_radius is not None else 'arc3,rad=0'  # None
         nx.draw_networkx_edges(
             G, self._pos,
             edgelist=edgelist,
@@ -394,7 +403,7 @@ class ConceptLatticeVisualizer(POSetVisualizer):
         """
         nodelist = list(range(len(self._lattice))) if nodelist is None else nodelist
         if not draw_bottom_concept:
-            nodelist.remove(self._lattice.bottom_concept_i)
+            nodelist.remove(self._lattice.bottom)
 
         if label_func is None:
             label_func = lambda c_i: self._concept_label_func(
