@@ -1,6 +1,6 @@
 from fcapy.context import FormalContext
 from fcapy.lattice import ConceptLattice
-from fcapy.visualizer import layouts
+from fcapy.visualizer import line_layouts
 from fcapy.visualizer import measures
 
 
@@ -13,10 +13,10 @@ def test_count_line_intersections():
     K = FormalContext.from_pandas(K_df)
     L = ConceptLattice.from_context(K)
 
-    pos_multipartite = layouts.multipartite_layout(L)
+    pos_multipartite = line_layouts.multipartite_layout(L)
     n_intersections_multipartite = measures.count_line_intersections(pos_multipartite, L)
     assert n_intersections_multipartite == 19, "Wrong number of line intersections in multipartite layout"
 
-    pos_fcart = layouts.fcart_layout(L)
+    pos_fcart = line_layouts.fcart_layout(L)
     n_intersections_fcart = measures.count_line_intersections(pos_fcart, L)
     assert n_intersections_fcart == 14, "Wrong number of line intersections in fcart layout"
